@@ -290,7 +290,7 @@
  *
  * All other bits should be set to zero.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -350,22 +350,63 @@ gss_init_sec_context (OM_uint32 * minor_status,
 		      gss_buffer_t output_token,
 		      OM_uint32 * ret_flags, OM_uint32 * time_rec)
 {
+  OM_uint32 maj_stat;
   _gss_mech_api_t mech;
+  int freecontext = 0;
 
-  mech = (*context_handle == GSS_C_NO_CONTEXT) ?
-    _gss_find_mech (mech_type) : _gss_find_mech ((*context_handle)->mech);
+  if (!context_handle)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_NO_CONTEXT | GSS_S_CALL_INACCESSIBLE_READ;
+    }
 
-  return mech->init_sec_context (minor_status,
-				 initiator_cred_handle,
-				 context_handle,
-				 target_name,
-				 mech_type,
-				 req_flags,
-				 time_req,
-				 input_chan_bindings,
-				 input_token,
-				 actual_mech_type,
-				 output_token, ret_flags, time_rec);
+  if (output_token == GSS_C_NO_BUFFER)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_FAILURE | GSS_S_CALL_BAD_STRUCTURE;
+    }
+
+  if (*context_handle == GSS_C_NO_CONTEXT)
+    mech = _gss_find_mech (mech_type);
+  else
+    mech = _gss_find_mech ((*context_handle)->mech);
+  if (mech == NULL)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_BAD_MECH;
+    }
+
+  if (actual_mech_type)
+    *actual_mech_type = mech->mech;
+
+  if (*context_handle == GSS_C_NO_CONTEXT)
+    {
+      *context_handle = xcalloc (sizeof (**context_handle), 1);
+      (*context_handle)->mech = mech->mech;
+      freecontext = 1;
+    }
+
+  maj_stat = mech->init_sec_context (minor_status,
+				     initiator_cred_handle,
+				     context_handle,
+				     target_name,
+				     mech_type,
+				     req_flags,
+				     time_req,
+				     input_chan_bindings,
+				     input_token,
+				     actual_mech_type,
+				     output_token, ret_flags, time_rec);
+  if (freecontext && GSS_ERROR (maj_stat))
+    {
+      free (*context_handle);
+      *context_handle = GSS_C_NO_CONTEXT;
+    }
+
+  return maj_stat;
 }
 
 /**
@@ -583,7 +624,7 @@ gss_init_sec_context (OM_uint32 * minor_status,
  *
  * All other bits should be set to zero.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_CONTINUE_NEEDED`: Indicates that a token from the peer
  * application is required to complete the context, and that
@@ -635,10 +676,20 @@ gss_accept_sec_context (OM_uint32 * minor_status,
 {
   _gss_mech_api_t mech;
 
+  if (!context_handle)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_NO_CONTEXT | GSS_S_CALL_INACCESSIBLE_READ;
+    }
+
   mech = (*context_handle == GSS_C_NO_CONTEXT) ?
     _gss_find_mech (mech_type ? *mech_type : GSS_C_NO_OID) :
     _gss_find_mech ((*context_handle)->mech);
 
+  if (mech_type)
+    *mech_type = mech->mech;
+
   return mech->accept_sec_context (minor_status,
 				   context_handle,
 				   acceptor_cred_handle,
@@ -692,7 +743,7 @@ gss_accept_sec_context (OM_uint32 * minor_status,
  * necessary, and that no token should be transferred by the
  * application.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -706,7 +757,14 @@ gss_delete_sec_context (OM_uint32 * minor_status,
   _gss_mech_api_t mech;
   OM_uint32 ret;
 
-  if (!context_handle || *context_handle == GSS_C_NO_CONTEXT)
+  if (!context_handle)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_NO_CONTEXT | GSS_S_CALL_INACCESSIBLE_READ;
+    }
+
+  if (*context_handle == GSS_C_NO_CONTEXT)
     {
       if (minor_status)
 	*minor_status = 0;
@@ -721,22 +779,15 @@ gss_delete_sec_context (OM_uint32 * minor_status,
 
   mech = _gss_find_mech ((*context_handle)->mech);
 
-  ret = mech->delete_sec_context (minor_status, context_handle, output_token);
-  if (GSS_ERROR (ret))
-    /* XXX? Ignore error. */ ;
+  ret = mech->delete_sec_context (NULL, context_handle, output_token);
 
-  (*context_handle)->peerptr = &(*context_handle)->peer;
-  ret = gss_release_name (minor_status, &(*context_handle)->peerptr);
-  if (GSS_ERROR (ret))
-    /* XXX? Ignore error. */ ;
+  if ((*context_handle)->peerptr != GSS_C_NO_NAME)
+    gss_release_name (NULL, &(*context_handle)->peerptr);
 
   free (*context_handle);
   *context_handle = GSS_C_NO_CONTEXT;
 
-  if (minor_status)
-    *minor_status = 0;
-
-  return GSS_S_COMPLETE;
+  return ret;
 }
 
 /**
@@ -762,7 +813,7 @@ gss_delete_sec_context (OM_uint32 * minor_status,
  * Gss_process_context_token provides a way to pass such a token to
  * the mechanism at any time.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -793,7 +844,7 @@ gss_process_context_token (OM_uint32 * minor_status,
  * Determines the number of seconds for which the specified context
  * will remain valid.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -918,7 +969,7 @@ gss_context_time (OM_uint32 * minor_status,
  * processes via a call to gss_export_sec_context().
  * - False - The security context is not transferable.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -977,7 +1028,7 @@ gss_inquire_context (OM_uint32 * minor_status,
  * should not return a value via max_input_bytes that is greater than
  * this length.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -1049,7 +1100,7 @@ gss_wrap_size_limit (OM_uint32 * minor_status,
  * providing it also sets the context_handle parameter to
  * GSS_C_NO_CONTEXT.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -1081,7 +1132,7 @@ gss_export_sec_context (OM_uint32 * minor_status,
  * another process.  A given interprocess token may be imported only
  * once.  See gss_export_sec_context.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
