@@ -95,7 +95,7 @@
  * gss_acquire_cred must return valid credential data, and may
  * therefore incur the overhead of a deferred credential acquisition.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -121,42 +121,54 @@ gss_acquire_cred (OM_uint32 * minor_status,
 		  gss_cred_id_t * output_cred_handle,
 		  gss_OID_set * actual_mechs, OM_uint32 * time_rec)
 {
-  OM_uint32 maj_stat;
   _gss_mech_api_t mech = NULL;
+  OM_uint32 maj_stat;
+
+  if (!output_cred_handle)
+    return GSS_S_NO_CRED | GSS_S_CALL_INACCESSIBLE_WRITE;
 
   if (desired_mechs != GSS_C_NO_OID_SET)
     {
-      int i;
-      int present;
+      size_t i;
 
-      /* XXX this assumes GSS credentials are mutually exclusive.
-         I.e., a credential for one mechanism cannot be used with
-         another mechanism.  If at some point in time this GSS library
-         support two different mechanisms that can use the same kind
-         of credential, this logic has to be improved somehow. */
+      /* Is the desired_mechs an OR or AND list?  I.e., if the OID set
+	 contain several OIDs, MUST the credential work with all of
+	 them?  Or just any of them?  The specification isn't entirely
+	 clear on this, to me.  This implement an OR list, chosing the
+	 first mechanism in the OID set we support.  We need more
+	 information in meta.c to implement AND lists. */
 
-      for (i = 0; _gss_mech_apis[i].mech; i++)
-	{
-	  maj_stat = gss_test_oid_set_member (minor_status,
-					      _gss_mech_apis[i].mech,
-					      desired_mechs, &present);
-	  if (!GSS_ERROR (maj_stat) && present)
-	    {
-	      mech = &_gss_mech_apis[i];
-	      break;
-	    }
-	}
+      for (i = 0; mech == NULL && i < desired_mechs->count; i++)
+	mech = _gss_find_mech ((&desired_mechs->elements)[i]);
     }
-
-  if (!mech)
+  else
     mech = _gss_find_mech (GSS_C_NO_OID);
 
-  return mech->acquire_cred (minor_status,
-			     desired_name,
-			     time_req,
-			     desired_mechs,
-			     cred_usage,
-			     output_cred_handle, actual_mechs, time_rec);
+  if (mech == NULL)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_BAD_MECH;
+    }
+
+  *output_cred_handle = xcalloc (sizeof (**output_cred_handle), 1);
+  (*output_cred_handle)->mech = mech->mech;
+
+  maj_stat = mech->acquire_cred (minor_status,
+				 desired_name,
+				 time_req,
+				 desired_mechs,
+				 cred_usage,
+				 output_cred_handle,
+				 actual_mechs, time_rec);
+  if (GSS_ERROR (maj_stat))
+    {
+      free (*output_cred_handle);
+      *output_cred_handle = GSS_C_NO_CREDENTIAL;
+      return maj_stat;
+    }
+
+  return GSS_S_COMPLETE;
 }
 
 /**
@@ -165,7 +177,7 @@ gss_acquire_cred (OM_uint32 * minor_status,
  * @input_cred_handle: (gss_cred_id_t, read, optional) The credential
  *   to which a credential-element will be added.  If
  *   GSS_C_NO_CREDENTIAL is specified, the routine will compose the
- *   new credential based on default behavior (see description above).
+ *   new credential based on default behavior (see text).
  *   Note that, while the credential-handle is not modified by
  *   gss_add_cred(), the underlying credential will be modified if
  *   output_credential_handle is NULL.
@@ -277,7 +289,7 @@ gss_acquire_cred (OM_uint32 * minor_status,
  * If GSS_C_NO_CREDENTIAL is specified as the input_cred_handle
  * parameter, a non-NULL output_cred_handle must be supplied.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -310,7 +322,7 @@ gss_add_cred (OM_uint32 * minor_status,
 	      gss_OID_set * actual_mechs,
 	      OM_uint32 * initiator_time_rec, OM_uint32 * acceptor_time_rec)
 {
-  return GSS_S_FAILURE;
+  return GSS_S_UNAVAILABLE;
 }
 
 /**
@@ -338,7 +350,7 @@ gss_add_cred (OM_uint32 * minor_status,
  *
  * Obtains information about a credential.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -358,13 +370,59 @@ gss_inquire_cred (OM_uint32 * minor_status,
 		  gss_cred_usage_t * cred_usage, gss_OID_set * mechanisms)
 {
   _gss_mech_api_t mech;
+  gss_cred_id_t local_cred_handle = GSS_C_NO_CREDENTIAL;
+  OM_uint32 maj_stat;
 
-  mech = cred_handle == GSS_C_NO_CREDENTIAL ?
-    _gss_find_mech (GSS_C_NO_OID) : _gss_find_mech (cred_handle->mech);
+  if (cred_handle == GSS_C_NO_CREDENTIAL)
+    {
+      gss_OID_set mechs;
 
-  return mech->inquire_cred (minor_status,
-			     cred_handle,
-			     name, lifetime, cred_usage, mechanisms);
+      maj_stat = gss_acquire_cred (minor_status,
+				   GSS_C_NO_NAME,
+				   GSS_C_INDEFINITE,
+				   GSS_C_NO_OID_SET,
+				   GSS_C_INITIATE,
+				   &local_cred_handle,
+				   &mechs,
+				   NULL);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
+
+      /* We assume all mechanisms the credential claim to support can
+	 handle the inquire_cred call. */
+      mech = _gss_find_mech (mechs->elements);
+
+      gss_release_oid_set (NULL, &mechs);
+
+      if (mech == NULL)
+	{
+	  if (minor_status)
+	    *minor_status = 0;
+	  return GSS_S_BAD_MECH;
+	}
+    }
+  else
+    {
+      mech = _gss_find_mech (cred_handle->mech);
+
+      if (mech == NULL)
+	{
+	  gss_warn ("gss_inquire_cred bug, report to %s", PACKAGE_BUGREPORT);
+	  if (minor_status)
+	    *minor_status = 0;
+	  return GSS_S_DEFECTIVE_CREDENTIAL;
+	}
+    }
+
+  maj_stat = mech->inquire_cred (minor_status,
+				 local_cred_handle != GSS_C_NO_CREDENTIAL ?
+				 local_cred_handle : cred_handle,
+				 name, lifetime, cred_usage, mechanisms);
+
+  if (local_cred_handle != GSS_C_NO_CREDENTIAL)
+    gss_release_cred (NULL, &local_cred_handle);
+
+  return maj_stat;
 }
 
 /**
@@ -402,7 +460,7 @@ gss_inquire_cred (OM_uint32 * minor_status,
  *
  * Obtains per-mechanism information about a credential.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -423,7 +481,55 @@ gss_inquire_cred_by_mech (OM_uint32 * minor_status,
 			  OM_uint32 * acceptor_lifetime,
 			  gss_cred_usage_t * cred_usage)
 {
-  return GSS_S_FAILURE;
+  _gss_mech_api_t mech;
+  gss_cred_id_t local_cred_handle = GSS_C_NO_CREDENTIAL;
+  OM_uint32 maj_stat;
+
+  if (mech_type == GSS_C_NO_OID)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_BAD_MECH;
+    }
+
+  mech = _gss_find_mech (mech_type);
+  if (mech == NULL)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_BAD_MECH;
+    }
+
+  if (cred_handle == GSS_C_NO_CREDENTIAL)
+    {
+      maj_stat = gss_acquire_cred (minor_status,
+				   GSS_C_NO_NAME,
+				   GSS_C_INDEFINITE,
+				   /* FIXME: We should create an OID
+				      set with mech_type and pass it
+				      as desired_mechs.  Maybe even
+				      check actual_mechs too. */
+				   GSS_C_NO_OID_SET,
+				   GSS_C_INITIATE,
+				   &local_cred_handle,
+				   NULL,
+				   NULL);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
+    }
+
+  maj_stat = mech->inquire_cred_by_mech
+    (minor_status,
+     local_cred_handle != GSS_C_NO_CREDENTIAL ?
+     local_cred_handle : cred_handle,
+     mech_type, name, initiator_lifetime, acceptor_lifetime, cred_usage);
+
+  if (local_cred_handle != GSS_C_NO_CREDENTIAL)
+    gss_release_cred (NULL, &local_cred_handle);
+
+  if (minor_status)
+    *minor_status = 0;
+  return maj_stat;
 }
 
 /**
@@ -435,11 +541,11 @@ gss_inquire_cred_by_mech (OM_uint32 * minor_status,
  *   nothing.
  *
  * Informs GSS-API that the specified credential handle is no longer
- * required by the application, and frees associated resources.
- * Implementations are encouraged to set the cred_handle to
- * GSS_C_NO_CREDENTIAL on successful completion of this call.
+ * required by the application, and frees associated resources.  The
+ * cred_handle is set to GSS_C_NO_CREDENTIAL on successful completion
+ * of this call.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -448,15 +554,37 @@ gss_inquire_cred_by_mech (OM_uint32 * minor_status,
 OM_uint32
 gss_release_cred (OM_uint32 * minor_status, gss_cred_id_t * cred_handle)
 {
-  if (minor_status)
-    *minor_status = 0;
+  _gss_mech_api_t mech;
+  OM_uint32 maj_stat;
 
-  if (cred_handle && *cred_handle)
-    free (*cred_handle);
+  if (!cred_handle)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_NO_CRED | GSS_S_CALL_INACCESSIBLE_READ;
+    }
 
-  /* XXX krb5 deallocate */
+  if (*cred_handle == GSS_C_NO_CREDENTIAL)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_COMPLETE;
+    }
 
+  mech = _gss_find_mech ((*cred_handle)->mech);
+  if (mech == NULL)
+    {
+      gss_warn ("gss_release_cred bug, report to %s", PACKAGE_BUGREPORT);
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_DEFECTIVE_CREDENTIAL;
+    }
+
+  maj_stat = mech->release_cred (minor_status, cred_handle);
+  free (*cred_handle);
   *cred_handle = GSS_C_NO_CREDENTIAL;
+  if (GSS_ERROR (maj_stat))
+    return maj_stat;
 
   return GSS_S_COMPLETE;
 }
