@@ -165,7 +165,7 @@ struct gss_status_codes gss_supplementary_errors[] = {
  * } while (message_context != 0);
  * ---------------------------------------------------
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -269,3 +269,20 @@ gss_display_status (OM_uint32 * minor_status,
 
   return GSS_S_COMPLETE;
 }
+
+/* For va_list, va_start, va_end. */
+#include <stdarg.h>
+
+void
+gss_warn (const char *format, ...)
+{
+  va_list ap;
+
+  va_start (ap, format);
+
+  fprintf (stderr, "%s: ", PACKAGE);
+  vfprintf (stderr, format, ap);
+  fprintf (stderr, "\n");
+
+  va_end (ap);
+}
