@@ -48,7 +48,7 @@
 
 typedef struct gss_name_struct
 {
-  int length;
+  size_t length;
   char *value;
   gss_OID type;
 } gss_name_desc;
@@ -64,7 +64,6 @@ typedef struct gss_cred_id_struct
 typedef struct gss_ctx_id_struct
 {
   gss_OID mech;
-  gss_name_desc peer;
   gss_name_t peerptr;
 #ifdef USE_KERBEROS5
   struct _gss_krb5_ctx_struct *krb5;
@@ -73,7 +72,7 @@ typedef struct gss_ctx_id_struct
 
 #define MAX_NT 5
 
-typedef struct _gss_mech_api_strict
+typedef struct _gss_mech_api_struct
 {
   gss_OID mech;
   gss_OID name_types[MAX_NT];
@@ -94,6 +93,9 @@ typedef struct _gss_mech_api_strict
     (OM_uint32 * minor_status,
      const gss_name_t input_name,
      const gss_OID mech_type, gss_name_t * output_name);
+  OM_uint32 (*export_name)
+    (OM_uint32 * minor_status,
+     const gss_name_t input_name, gss_buffer_t exported_name);
     OM_uint32 (*wrap)
     (OM_uint32 * minor_status,
      const gss_ctx_id_t context_handle, int conf_req_flag,
@@ -129,6 +131,8 @@ typedef struct _gss_mech_api_strict
      gss_cred_usage_t cred_usage,
      gss_cred_id_t * output_cred_handle,
      gss_OID_set * actual_mechs, OM_uint32 * time_rec);
+  OM_uint32 (*release_cred)
+    (OM_uint32 * minor_status, gss_cred_id_t * cred_handle);
     OM_uint32 (*accept_sec_context)
     (OM_uint32 * minor_status,
      gss_ctx_id_t * context_handle,
@@ -152,10 +156,19 @@ typedef struct _gss_mech_api_strict
      const gss_cred_id_t cred_handle,
      gss_name_t * name, OM_uint32 * lifetime,
      gss_cred_usage_t * cred_usage, gss_OID_set * mechanisms);
+  OM_uint32 (*inquire_cred_by_mech)
+    (OM_uint32 * minor_status,
+     const gss_cred_id_t cred_handle,
+     const gss_OID mech_type,
+     gss_name_t * name,
+     OM_uint32 * initiator_lifetime,
+     OM_uint32 * acceptor_lifetime,
+     gss_cred_usage_t * cred_usage);
 } _gss_mech_api_desc, *_gss_mech_api_t;
 
-extern _gss_mech_api_desc _gss_mech_apis[];
+_gss_mech_api_t _gss_find_mech (const gss_OID oid);
 
-_gss_mech_api_t _gss_find_mech (gss_OID oid);
+/* See error.c. */
+extern void gss_warn (const char *format, ...);
 
 #endif /* _INTERNAL_H */
