@@ -22,14 +22,14 @@
 /* Get specification. */
 #include "k5internal.h"
 
-OM_uint32
-gss_krb5_acquire_cred1 (OM_uint32 * minor_status,
-			const gss_name_t desired_name,
-			OM_uint32 time_req,
-			const gss_OID_set desired_mechs,
-			gss_cred_usage_t cred_usage,
-			gss_cred_id_t * output_cred_handle,
-			gss_OID_set * actual_mechs, OM_uint32 * time_rec)
+static OM_uint32
+acquire_cred1 (OM_uint32 * minor_status,
+	       const gss_name_t desired_name,
+	       OM_uint32 time_req,
+	       const gss_OID_set desired_mechs,
+	       gss_cred_usage_t cred_usage,
+	       gss_cred_id_t * output_cred_handle,
+	       gss_OID_set * actual_mechs, OM_uint32 * time_rec)
 {
   _gss_krb5_cred_t k5 = (*output_cred_handle)->krb5;
   OM_uint32 maj_stat;
@@ -47,16 +47,8 @@ gss_krb5_acquire_cred1 (OM_uint32 * minor_status,
 	return maj_stat;
     }
 
-  if (gss_oid_equal (desired_name->type, GSS_KRB5_NT_PRINCIPAL_NAME))
-    {
-      maj_stat = gss_duplicate_name (minor_status, desired_name,
-				     &k5->peerptr);
-    }
-  else
-    {
-      maj_stat = gss_krb5_canonicalize_name (minor_status, desired_name,
-					     GSS_KRB5, &k5->peerptr);
-    }
+  maj_stat = gss_krb5_canonicalize_name (minor_status, desired_name,
+					 GSS_KRB5, &k5->peerptr);
   if (GSS_ERROR (maj_stat))
     return maj_stat;
 
@@ -79,7 +71,7 @@ gss_krb5_acquire_cred1 (OM_uint32 * minor_status,
     {
       if (minor_status)
 	*minor_status = GSS_KRB5_S_KG_KEYTAB_NOMATCH;
-      return GSS_S_FAILURE;
+      return GSS_S_NO_CRED;
     }
 
   if (time_rec)
@@ -98,44 +90,78 @@ gss_krb5_acquire_cred (OM_uint32 * minor_status,
 		       gss_OID_set * actual_mechs, OM_uint32 * time_rec)
 {
   OM_uint32 maj_stat;
-  gss_cred_id_t p;
-
-  if (minor_status)
-    *minor_status = 0;
+  gss_cred_id_t p = *output_cred_handle;
 
   if (actual_mechs)
     {
       maj_stat = gss_create_empty_oid_set (minor_status, actual_mechs);
       if (GSS_ERROR (maj_stat))
 	return maj_stat;
-      maj_stat =
-	gss_add_oid_set_member (minor_status, GSS_KRB5, actual_mechs);
+      maj_stat = gss_add_oid_set_member (minor_status, GSS_KRB5,
+					 actual_mechs);
       if (GSS_ERROR (maj_stat))
 	return maj_stat;
     }
 
-  p = xcalloc (sizeof (*p), 1);
-  p->mech = GSS_KRB5;
   p->krb5 = xcalloc (sizeof (*p->krb5), 1);
 
-  maj_stat = gss_krb5_acquire_cred1 (minor_status, desired_name, time_req,
-				     desired_mechs, cred_usage,
-				     &p, actual_mechs, time_rec);
+  maj_stat = acquire_cred1 (minor_status, desired_name, time_req,
+			    desired_mechs, cred_usage,
+			    &p, actual_mechs, time_rec);
   if (GSS_ERROR (maj_stat))
     {
-      OM_uint32 junk;
-
-      gss_release_oid_set (&junk, actual_mechs);
-
+      if (actual_mechs)
+	gss_release_oid_set (NULL, actual_mechs);
       free (p->krb5);
-      free (p);
-      *output_cred_handle = NULL;
 
       return maj_stat;
     }
 
-  *output_cred_handle = p;
+  if (minor_status)
+    *minor_status = 0;
+  return GSS_S_COMPLETE;
+}
 
+static OM_uint32
+inquire_cred (OM_uint32 * minor_status,
+	      const gss_cred_id_t cred_handle,
+	      gss_name_t * name,
+	      OM_uint32 * lifetime,
+	      gss_cred_usage_t * cred_usage,
+	      gss_OID_set * mechanisms)
+{
+  OM_uint32 maj_stat;
+
+  if (cred_handle == GSS_C_NO_CREDENTIAL)
+    return GSS_S_NO_CRED;
+
+  if (mechanisms)
+    {
+      maj_stat = gss_create_empty_oid_set (minor_status, mechanisms);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
+      maj_stat = gss_add_oid_set_member (minor_status, GSS_KRB5,
+					 mechanisms);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
+    }
+
+  if (name)
+    {
+      maj_stat = gss_duplicate_name (minor_status, cred_handle->krb5->peerptr,
+				     name);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
+    }
+
+  if (cred_usage)
+    *cred_usage = GSS_C_BOTH;
+
+  if (lifetime)
+    *lifetime = GSS_C_INDEFINITE;
+
+  if (minor_status)
+    *minor_status = 0;
   return GSS_S_COMPLETE;
 }
 
@@ -146,40 +172,39 @@ gss_krb5_inquire_cred (OM_uint32 * minor_status,
 		       OM_uint32 * lifetime,
 		       gss_cred_usage_t * cred_usage,
 		       gss_OID_set * mechanisms)
+{
+  return inquire_cred (minor_status, cred_handle, name, lifetime,
+		       cred_usage, mechanisms);
+}
+
+OM_uint32
+gss_krb5_inquire_cred_by_mech (OM_uint32 * minor_status,
+			       const gss_cred_id_t cred_handle,
+			       const gss_OID mech_type,
+			       gss_name_t * name,
+			       OM_uint32 * initiator_lifetime,
+			       OM_uint32 * acceptor_lifetime,
+			       gss_cred_usage_t * cred_usage)
 {
   OM_uint32 maj_stat;
 
+  maj_stat = inquire_cred (minor_status, cred_handle, name,
+			   initiator_lifetime, cred_usage, NULL);
+
+  if (acceptor_lifetime)
+    *acceptor_lifetime = *initiator_lifetime;
+
+  return maj_stat;
+}
+
+OM_uint32
+gss_krb5_release_cred (OM_uint32 * minor_status,
+		       gss_cred_id_t * cred_handle)
+{
+  shishi_done ((*cred_handle)->krb5->sh);
+  free ((*cred_handle)->krb5);
+
   if (minor_status)
     *minor_status = 0;
-
-  if (cred_handle == GSS_C_NO_CREDENTIAL)
-    {
-
-    }
-
-  if (name)
-    {
-      maj_stat = gss_duplicate_name (minor_status, cred_handle->krb5->peerptr,
-				     name);
-      if (GSS_ERROR (maj_stat))
-	return maj_stat;
-    }
-
-  if (lifetime)
-    *lifetime = gss_krb5_tktlifetime (cred_handle->krb5->tkt);
-
-  if (cred_usage)
-    *cred_usage = GSS_C_BOTH;
-
-  if (mechanisms)
-    {
-      maj_stat = gss_create_empty_oid_set (minor_status, mechanisms);
-      if (GSS_ERROR (maj_stat))
-	return maj_stat;
-      maj_stat = gss_add_oid_set_member (minor_status, GSS_KRB5, mechanisms);
-      if (GSS_ERROR (maj_stat))
-	return maj_stat;
-    }
-
   return GSS_S_COMPLETE;
 }
