@@ -30,6 +30,26 @@
 		  ((buf[2] & 0xFF) << 16) |	\
 		  ((buf[3] & 0xFF) << 24))
 
+OM_uint32
+gss_krb5_get_mic (OM_uint32 * minor_status,
+		  const gss_ctx_id_t context_handle,
+		  gss_qop_t qop_req,
+		  const gss_buffer_t message_buffer,
+		  gss_buffer_t message_token)
+{
+  return GSS_S_UNAVAILABLE;
+}
+
+OM_uint32
+gss_krb5_verify_mic (OM_uint32 * minor_status,
+		     const gss_ctx_id_t context_handle,
+		     const gss_buffer_t message_buffer,
+		     const gss_buffer_t token_buffer,
+		     gss_qop_t * qop_state)
+{
+  return GSS_S_UNAVAILABLE;
+}
+
 OM_uint32
 gss_krb5_wrap (OM_uint32 * minor_status,
 	       const gss_ctx_id_t context_handle,
