@@ -21,153 +21,6 @@
 
 #include "internal.h"
 
-/**
- * gss_oid_equal:
- * @first_oid: (Object ID, read) First Object identifier.
- * @second_oid: (Object ID, read) First Object identifier.
- *
- * Compare two OIDs for equality.  The comparison is "deep", i.e., the
- * actual byte sequences of the OIDs are compared instead of just the
- * pointer equality.
- *
- * WARNING: This function is a GNU GSSLib specific extension, and is
- * not part of the official GSS API.
- *
- * Return value: Returns boolean value true when the two OIDs are
- *   equal, otherwise false.
- **/
-int
-gss_oid_equal (gss_OID first_oid, gss_OID second_oid)
-{
-
-  return first_oid && second_oid &&
-    first_oid->length == second_oid->length &&
-    memcmp (first_oid->elements, second_oid->elements,
-	    second_oid->length) == 0;
-}
-
-/**
- * gss_copy_oid:
- * @minor_status: (integer, modify) Mechanism specific status code.
- * @src_oid: (Object ID, read) The object identifier to copy.
- * @dest_oid: (Object ID, modify) The resultant copy of @src_oid.
- *   Storage associated with this name must be freed by the
- *   application, but gss_release_oid() cannot be used generally as it
- *   deallocate the the oid structure itself too (use
- *   gss_duplicate_oid() if you don't want this problem.)
- *
- * Make an exact copy of the given OID, that shares no memory areas
- * with the original.
- *
- * WARNING: This function is a GNU GSSLib specific extension, and is
- * not part of the official GSS API.
- *
- * Valid return values and their meaning:
- *
- * `GSS_S_COMPLETE`: Successful completion.
- **/
-OM_uint32
-gss_copy_oid (OM_uint32 * minor_status,
-	      const gss_OID src_oid, gss_OID dest_oid)
-{
-  if (minor_status)
-    *minor_status = 0;
-
-  if (!src_oid)
-    return GSS_S_FAILURE | GSS_S_CALL_INACCESSIBLE_READ;
-
-  if (src_oid->length == 0 || src_oid->elements == NULL)
-    return GSS_S_FAILURE | GSS_S_CALL_BAD_STRUCTURE;
-
-  dest_oid->length = src_oid->length;
-  dest_oid->elements = xmalloc (src_oid->length);
-  memcpy (dest_oid->elements, src_oid->elements, src_oid->length);
-
-  return GSS_S_COMPLETE;
-}
-
-/**
- * gss_duplicate_oid:
- * @minor_status: (integer, modify) Mechanism specific status code.
- * @src_oid: (Object ID, read) The object identifier to duplicate.
- * @dest_oid: (Object ID, modify) The resultant copy of @src_oid.
- *   Storage associated with this name must be freed by the
- *   application, by calling gss_release_oid().
- *
- * Allocate a new OID and make it an exact copy of the given OID, that
- * shares no memory areas with the original.
- *
- * WARNING: This function is a GNU GSSLib specific extension, and is
- * not part of the official GSS API.
- *
- * Valid return values and their meaning:
- *
- * `GSS_S_COMPLETE`: Successful completion.
- **/
-OM_uint32
-gss_duplicate_oid (OM_uint32 * minor_status,
-		   const gss_OID src_oid, gss_OID * dest_oid)
-{
-  /* This function is not part of the official GSS API */
-
-  OM_uint32 maj_stat;
-
-  if (minor_status)
-    *minor_status = 0;
-
-  if (!src_oid)
-    return GSS_S_FAILURE | GSS_S_CALL_INACCESSIBLE_READ;
-
-  if (src_oid->length == 0 || src_oid->elements == NULL)
-    return GSS_S_FAILURE | GSS_S_CALL_BAD_STRUCTURE;
-
-  *dest_oid = xmalloc (sizeof (**dest_oid));
-
-  maj_stat = gss_copy_oid (minor_status, src_oid, *dest_oid);
-  if (GSS_ERROR (maj_stat))
-    {
-      free (*dest_oid);
-      return maj_stat;
-    }
-
-  return GSS_S_COMPLETE;
-}
-
-/**
- * gss_release_oid:
- * @minor_status: (Integer, modify) Mechanism specific status code.
- * @oid: (gss_name_t, modify) The OID to be deleted.
- *
- * Free GSSAPI-allocated storage associated with an OID.  The OID is
- * set to GSS_C_NO_NAME on successful completion of this call.
- *
- * WARNING: This function is a GNU GSSLib specific extension, and is
- * not part of the official GSS API.
- *
- * Valid return values and their meaning:
- *
- * `GSS_S_COMPLETE`: Successful completion.
- **/
-OM_uint32
-gss_release_oid (OM_uint32 * minor_status, gss_OID *oid)
-{
-  if (minor_status)
-    *minor_status = 0;
-
-  if (!oid)
-    return GSS_S_FAILURE | GSS_S_CALL_INACCESSIBLE_READ;
-
-  if (*oid != GSS_C_NO_OID)
-    {
-      if ((*oid)->elements)
-	free ((*oid)->elements);
-      free (*oid);
-      *oid = GSS_C_NO_OID;
-    }
-
-  return GSS_S_COMPLETE;
-}
-
 /**
  * gss_create_empty_oid_set:
  * @minor_status: (integer, modify) Mechanism specific status code.
@@ -182,7 +35,7 @@ gss_release_oid (OM_uint32 * minor_status, gss_OID *oid)
  * be used to construct sets of mechanism object identifiers, for
  * input to gss_acquire_cred.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -220,7 +73,7 @@ gss_create_empty_oid_set (OM_uint32 * minor_status, gss_OID_set * oid_set)
  * not already contained within the elements array; if the member_oid
  * is already present, the oid_set should remain unchanged.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -239,7 +92,7 @@ gss_add_oid_set_member (OM_uint32 * minor_status,
 
   major_stat = gss_test_oid_set_member (minor_status, member_oid,
 					*oid_set, &present);
-  if (major_stat != GSS_S_COMPLETE)
+  if (GSS_ERROR (major_stat))
     return major_stat;
 
   if (present)
@@ -255,7 +108,7 @@ gss_add_oid_set_member (OM_uint32 * minor_status,
 
   major_stat = gss_copy_oid (minor_status, member_oid,
 			     (*oid_set)->elements + ((*oid_set)->count - 1));
-  if (major_stat != GSS_S_COMPLETE)
+  if (GSS_ERROR (major_stat))
     return major_stat;
 
   return GSS_S_COMPLETE;
@@ -276,7 +129,7 @@ gss_add_oid_set_member (OM_uint32 * minor_status,
  * gss_acquire_cred(), and gss_inquire_cred(), but will also work with
  * user-generated sets.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -324,7 +177,7 @@ gss_test_oid_set_member (OM_uint32 * minor_status,
  * The gss_OID_set parameter is set to GSS_C_NO_OID_SET on successful
  * completion of this routine.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -349,6 +202,10 @@ gss_release_oid_set (OM_uint32 * minor_status, gss_OID_set * set)
   return GSS_S_COMPLETE;
 }
 
+/* See meta.c. */
+OM_uint32
+_gss_indicate_mechs1 (OM_uint32 * minor_status, gss_OID_set *mech_set);
+
 /**
  * gss_indicate_mechs:
  * @minor_status: (integer, modify) Mechanism specific status code.
@@ -361,7 +218,7 @@ gss_release_oid_set (OM_uint32 * minor_status, gss_OID_set * set)
  * Allows an application to determine which underlying security
  * mechanisms are available.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -369,21 +226,16 @@ OM_uint32
 gss_indicate_mechs (OM_uint32 * minor_status, gss_OID_set * mech_set)
 {
   OM_uint32 maj_stat;
-  int i;
 
   maj_stat = gss_create_empty_oid_set (minor_status, mech_set);
-  if (maj_stat != GSS_S_COMPLETE)
+  if (GSS_ERROR (maj_stat))
     return maj_stat;
 
-  for (i = 0; _gss_mech_apis[i].mech; i++)
+  maj_stat = _gss_indicate_mechs1 (minor_status, mech_set);
+  if (GSS_ERROR (maj_stat))
     {
-      maj_stat = gss_add_oid_set_member (minor_status, _gss_mech_apis[i].mech,
-					 mech_set);
-      if (maj_stat != GSS_S_COMPLETE)
-	{
-	  gss_release_oid_set (minor_status, mech_set);
-	  return maj_stat;
-	}
+      gss_release_oid_set (NULL, mech_set);
+      return maj_stat;
     }
 
   if (minor_status)
@@ -407,7 +259,7 @@ gss_indicate_mechs (OM_uint32 * minor_status, gss_OID_set * mech_set)
  * GSS-API routine may be passed to gss_release_buffer (even if there
  * is no storage associated with the buffer).
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -421,6 +273,7 @@ gss_release_buffer (OM_uint32 * minor_status, gss_buffer_t buffer)
     {
       if (buffer->value)
 	free (buffer->value);
+      buffer->value = NULL;
       buffer->length = 0;
     }
 
