@@ -43,7 +43,7 @@
  * which case the returned internal name will be an MN for the
  * mechanism that exported the name.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -114,7 +114,7 @@ gss_import_name (OM_uint32 * minor_status,
  * conversion between name types may return GSS_C_NO_OID via the
  * output_name_type parameter.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -162,7 +162,7 @@ gss_display_name (OM_uint32 * minor_status,
  * principal, the routines should indicate that the two names do not
  * refer to the same identity.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -200,7 +200,7 @@ gss_compare_name (OM_uint32 * minor_status,
  * name.  The name is set to GSS_C_NO_NAME on successful completion of
  * this call.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -209,12 +209,11 @@ gss_compare_name (OM_uint32 * minor_status,
 OM_uint32
 gss_release_name (OM_uint32 * minor_status, gss_name_t * name)
 {
+  if (minor_status)
+    *minor_status = 0;
+
   if (!name)
-    {
-      if (minor_status)
-	*minor_status = 0;
-      return GSS_S_BAD_NAME | GSS_S_CALL_INACCESSIBLE_READ;
-    }
+    return GSS_S_BAD_NAME | GSS_S_CALL_INACCESSIBLE_READ;
 
   if (*name != GSS_C_NO_NAME)
     {
@@ -238,7 +237,7 @@ gss_release_name (OM_uint32 * minor_status, gss_name_t * name)
  *
  * Returns the set of nametypes supported by the specified mechanism.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -272,57 +271,58 @@ gss_inquire_names_for_mech (OM_uint32 * minor_status,
   return GSS_S_COMPLETE;
 }
 
+/* Add mechanism MECH to OID set MECH_TYPES if mechanism MECH support
+   the NAME_TYPE name type. */
 static OM_uint32
-_gss_inquire_mechs_for_name2 (OM_uint32 * minor_status,
-			      _gss_mech_api_t mech,
-			      gss_OID name_type, gss_OID_set * mech_types)
+_gss_inquire_mechs_for_name3 (OM_uint32 * minor_status,
+			      gss_OID mech,
+			      gss_OID name_type,
+			      gss_OID_set * mech_types)
 {
   gss_OID_set oids;
   int supported;
   OM_uint32 maj_stat;
 
-  maj_stat = gss_inquire_names_for_mech (minor_status, mech->mech, &oids);
-  if (maj_stat != GSS_S_COMPLETE)
+  maj_stat = gss_inquire_names_for_mech (minor_status, mech, &oids);
+  if (GSS_ERROR (maj_stat))
     return maj_stat;
 
   maj_stat = gss_test_oid_set_member (minor_status, name_type,
 				      oids, &supported);
-  if (maj_stat != GSS_S_COMPLETE)
-    {
-      gss_release_oid_set (minor_status, &oids);
-      return maj_stat;
-    }
+  gss_release_oid_set (minor_status, &oids);
+  if (GSS_ERROR (maj_stat))
+    return maj_stat;
 
   if (supported)
     {
-      maj_stat = gss_add_oid_set_member (minor_status,
-					 mech->mech, mech_types);
-      if (maj_stat != GSS_S_COMPLETE)
-	{
-	  gss_release_oid_set (minor_status, &oids);
-	  return maj_stat;
-	}
+      maj_stat = gss_add_oid_set_member (minor_status, mech, mech_types);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
     }
-  gss_release_oid_set (minor_status, &oids);
 
   if (minor_status)
     *minor_status = 0;
   return GSS_S_COMPLETE;
 }
 
+/* Iterate over SUPPORTED_MECH_TYPES and invoke
+   gss_inquire_mechs_for_name3 on each type, thus adding all mechanism
+   OIDs, that support the NAME_TYPE name type, to OUT_MECH_TYPES. */
 static OM_uint32
-_gss_inquire_mechs_for_name1 (OM_uint32 * minor_status,
-			      gss_OID name_type, gss_OID_set * mech_types)
+_gss_inquire_mechs_for_name2 (OM_uint32 * minor_status,
+			      gss_OID name_type,
+			      gss_OID_set * out_mech_types,
+			      gss_OID_set supported_mech_types)
 {
   OM_uint32 maj_stat;
-  int i;
+  size_t i;
 
-  for (i = 0; _gss_mech_apis[i].mech; i++)
+  for (i = 0; i < supported_mech_types->count; i++)
     {
-      maj_stat = _gss_inquire_mechs_for_name2 (minor_status,
-					       &_gss_mech_apis[i],
-					       name_type, mech_types);
-      if (maj_stat != GSS_S_COMPLETE)
+      maj_stat = _gss_inquire_mechs_for_name3
+	(minor_status, &(supported_mech_types->elements)[i],
+	 name_type, out_mech_types);
+      if (GSS_ERROR(maj_stat))
 	return maj_stat;
     }
 
@@ -331,6 +331,33 @@ _gss_inquire_mechs_for_name1 (OM_uint32 * minor_status,
   return GSS_S_COMPLETE;
 }
 
+/* List all supported mechanisms, and invoke
+   gss_inquire_mechs_for_name2, thus adding all mechanism OIDs, that
+   support the NAME_TYPE name type, to OUT_MECH_TYPES. */
+static OM_uint32
+_gss_inquire_mechs_for_name1 (OM_uint32 * minor_status,
+			      gss_OID name_type, gss_OID_set * out_mech_types)
+{
+  OM_uint32 maj_stat;
+  gss_OID_set supported_mech_types;
+  size_t i;
+
+  maj_stat = gss_indicate_mechs (minor_status, &supported_mech_types);
+  if (GSS_ERROR (maj_stat))
+    return maj_stat;
+
+  maj_stat = _gss_inquire_mechs_for_name2 (minor_status, name_type,
+					   out_mech_types,
+					   supported_mech_types);
+  gss_release_oid_set (minor_status, &supported_mech_types);
+  if (GSS_ERROR (maj_stat))
+    return maj_stat;
+
+  if (minor_status)
+    *minor_status = 0;
+  return GSS_S_COMPLETE;
+}
+
 /**
  * gss_inquire_mechs_for_name:
  * @minor_status: (Integer, modify) Mechanism specific status code.
@@ -356,7 +383,7 @@ _gss_inquire_mechs_for_name1 (OM_uint32 * minor_status,
  * to the name type).  Thus this routine should be used only as a
  * prefilter for a call to a subsequent mechanism-specific routine.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -372,13 +399,17 @@ gss_inquire_mechs_for_name (OM_uint32 * minor_status,
 {
   OM_uint32 maj_stat;
 
+  if (input_name == GSS_C_NO_NAME)
+    return GSS_S_BAD_NAME | GSS_S_CALL_INACCESSIBLE_READ;
+
   maj_stat = gss_create_empty_oid_set (minor_status, mech_types);
-  if (maj_stat != GSS_S_COMPLETE)
+  if (GSS_ERROR (maj_stat))
     return maj_stat;
 
-  maj_stat = _gss_inquire_mechs_for_name1 (minor_status, input_name->type,
+  maj_stat = _gss_inquire_mechs_for_name1 (minor_status,
+					   input_name->type,
 					   mech_types);
-  if (maj_stat != GSS_S_COMPLETE)
+  if (GSS_ERROR (maj_stat))
     {
       gss_release_oid_set (minor_status, mech_types);
       return maj_stat;
@@ -405,7 +436,7 @@ gss_inquire_mechs_for_name (OM_uint32 * minor_status,
  * a valid MN (i.e. an internal name generated by
  * gss_accept_sec_context() or by gss_canonicalize_name()).
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
@@ -421,7 +452,35 @@ OM_uint32
 gss_export_name (OM_uint32 * minor_status,
 		 const gss_name_t input_name, gss_buffer_t exported_name)
 {
-  return GSS_S_FAILURE;
+  OM_uint32 maj_stat;
+  gss_OID_set mechs;
+  _gss_mech_api_t mech;
+
+  maj_stat = gss_inquire_mechs_for_name (minor_status, input_name, &mechs);
+  if (GSS_ERROR (maj_stat))
+    return maj_stat;
+
+  if (mechs->count == 0)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_BAD_NAMETYPE;
+    }
+
+  /* We just select a random mechanism that support this name-type.
+     I'm not sure how we can be more predicatable, given the
+     definition of this function. */
+
+  mech = _gss_find_mech (mechs->elements);
+  if (mech == NULL)
+    {
+      gss_warn ("gss_export_name bug, please report to %s", PACKAGE_BUGREPORT);
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_BAD_MECH;
+    }
+
+  return mech->export_name (minor_status, input_name, exported_name);
 }
 
 /**
@@ -444,7 +503,7 @@ gss_export_name (OM_uint32 * minor_status,
  * its only member, followed by a call to gss_init_sec_context(),
  * specifying @mech_type as the authentication mechanism.
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  **/
@@ -456,6 +515,12 @@ gss_canonicalize_name (OM_uint32 * minor_status,
   _gss_mech_api_t mech;
 
   mech = _gss_find_mech (mech_type);
+  if (mech == NULL)
+    {
+      if (minor_status)
+	*minor_status = 0;
+      return GSS_S_BAD_MECH;
+    }
 
   return mech->canonicalize_name (minor_status, input_name,
 				  mech_type, output_name);
@@ -474,7 +539,7 @@ gss_canonicalize_name (OM_uint32 * minor_status,
  * and dest_name must both be released, and the release of one shall
  * not affect the validity of the other).
  *
- * Valid return values and their meaning:
+ * Return value:
  *
  * `GSS_S_COMPLETE`: Successful completion.
  *
