@@ -19,7 +19,123 @@
  *
  */
 
-#include "utils.c"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <stdarg.h>
+#include <ctype.h>
+#include <string.h>
+
+/* i18n. */
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#else
+# define setlocale(Category, Locale)	/* empty */
+#endif
+#include "gettext.h"
+
+/* Get GSS prototypes. */
+#include "api.h"
+#include "ext.h"
+#ifdef USE_KERBEROS5
+# include "krb5.h"
+#endif
+
+static int verbose = 0;
+static int debug = 0;
+static int error_count = 0;
+static int break_on_error = 0;
+
+static void
+fail (const char *format, ...)
+{
+  va_list arg_ptr;
+
+  va_start (arg_ptr, format);
+  vfprintf (stderr, format, arg_ptr);
+  va_end (arg_ptr);
+  error_count++;
+  if (break_on_error)
+    exit (1);
+}
+
+static void
+success (const char *format, ...)
+{
+  va_list arg_ptr;
+
+  va_start (arg_ptr, format);
+  if (verbose)
+    vfprintf (stdout, format, arg_ptr);
+  va_end (arg_ptr);
+}
+
+static void
+escapeprint (const unsigned char *str, int len)
+{
+  int i;
+
+  if (!str || !len)
+    return;
+
+  printf ("\t ;; `");
+  for (i = 0; i < len; i++)
+    if ((str[i] >= 'A' && str[i] <= 'Z') ||
+	(str[i] >= 'a' && str[i] <= 'z') ||
+	(str[i] >= '0' && str[i] <= '9') || str[i] == '.')
+      printf ("%c", str[i]);
+    else
+      printf ("\\x%02x", str[i]);
+  printf ("' (length %d bytes)\n", len);
+}
+
+static void
+hexprint (const unsigned char *str, int len)
+{
+  int i;
+
+  if (!str || !len)
+    return;
+
+  printf ("\t ;; ");
+  for (i = 0; i < len; i++)
+    {
+      printf ("%02x ", str[i]);
+      if ((i + 1) % 8 == 0)
+	printf (" ");
+      if ((i + 1) % 16 == 0 && i + 1 < len)
+	printf ("\n\t ;; ");
+    }
+}
+
+static void
+binprint (const unsigned char *str, int len)
+{
+  int i;
+
+  if (!str || !len)
+    return;
+
+  printf ("\t ;; ");
+  for (i = 0; i < len; i++)
+    {
+      printf ("%d%d%d%d%d%d%d%d ",
+	      str[i] & 0x80 ? 1 : 0,
+	      str[i] & 0x40 ? 1 : 0,
+	      str[i] & 0x20 ? 1 : 0,
+	      str[i] & 0x10 ? 1 : 0,
+	      str[i] & 0x08 ? 1 : 0,
+	      str[i] & 0x04 ? 1 : 0,
+	      str[i] & 0x02 ? 1 : 0, str[i] & 0x01 ? 1 : 0);
+      if ((i + 1) % 3 == 0)
+	printf (" ");
+      if ((i + 1) % 6 == 0 && i + 1 < len)
+	printf ("\n\t ;; ");
+    }
+}
 
 int
 main (int argc, char *argv[])
@@ -30,9 +146,7 @@ main (int argc, char *argv[])
   gss_OID_set oids;
   int n;
 
-#if HAVE_LOCALE_H
   setlocale (LC_ALL, "");
-#endif
   bindtextdomain (PACKAGE, LOCALEDIR);
 
   do
